use master
go

-- each database has its own store of keys, certs
-- only master has Service Master Key

select * from sys.symmetric_keys
go

select * from sys.asymmetric_keys
go

select * from sys.certificates
go

-- you can back up the key to a file
backup service master key to file = 'c:\temp\masterkey2.bkp' 
  encryption by password = 'P@ssw0rd'
go

-- knows they are the same
-- FORCE option ignores errors in reencryption
restore service master key from file = 'c:\temp\masterkey2.bkp' 
  decryption by password = 'P@ssw0rd'
go

-- regenerate the service master key
alter service master key regenerate 
go

create database secdemo
go

use secdemo
go

create master key encryption by password = 'pass1Word'
go

select * from sys.symmetric_keys
go

-- is_master_key_encrypted_by_server
select name, is_master_key_encrypted_by_server  from sys.databases
go

-- you can drop encryption by server, but...
-- you cannot drop encryption by password
alter master key drop encryption by password = 'pass1Word'
go

-- you can have two passwords
alter master key add encryption by password = 'pass2Word'
go

-- after this, its encrypted by only pass2Word
alter master key drop encryption by password = 'pass1Word'
go

-- you can drop encryption by service master key
-- after this, you must open the master key to use it
alter master key drop encryption by service master key
go

create certificate foo2 with subject = 'foo2'

-- if no encryption by service master key
-- you must open the master key before using it
alter master key add encryption by service master key
go

-- this works. open then use
open master key decryption by password = 'pass2Word'
alter master key add encryption by service master key
go

close all symmetric keys

-- after removing service master key and detach-attach
alter master key regenerate with encryption by password = 'pass2Word'
go

backup master key to file = 'c:\temp\secdemo_master_key.key'
 encryption by password = 'StrongPW1'

-- the password for the master key when it was backed up 
-- was 'pass2Word'
-- now, its 'StrongKeyPW1'
restore master key from file = 'c:\temp\secdemo_master_key.key'
 decryption by password = 'StrongPW1'
 encryption by password = 'StrongKeyPW1'


-- cleanup, if needed
use master
go
drop database secdemo
go
